@@ -23,9 +23,11 @@
 #ifndef __SEPOL_UTIL_H__
 #define __SEPOL_UTIL_H__
 
-#include <sys/cdefs.h>
+#include <sepol/policydb/policydb.h>
 
-__BEGIN_DECLS
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 extern int add_i_to_a(uint32_t i, uint32_t * cnt, uint32_t ** a);
 
@@ -40,5 +42,8 @@ char *sepol_extended_perms_to_string(avtab_extended_perms_t *xperms);
  */
 extern int tokenize(char *line_buf, char delim, int num_args, ...);
 
-__END_DECLS
+#ifdef __cplusplus
+}
+#endif
+
 #endif
